// $Id: CpuWidget.cpp,v 1.63 2014/02/10 10:07:45 hupereir Exp $

/******************************************************************************
*
* Copyright (C) 2002 Hugo PEREIRA <mailto: hugo.pereira@free.fr>
*
* This is free software; you can redistribute it and/or modify it under the
* terms of the GNU General Public License as published by the Free Software
* Foundation; either version 2 of the License, or (at your option) any later
* version.
*
* This software is distributed in the hope that it will be useful, but WITHOUT
* Any WARRANTY; without even the implied warranty of MERCHANTABILITY or
* FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
* for more details.
*
* You should have received a copy of the GNU General Public License along with
* this program.  If not, see <http://www.gnu.org/licenses/>.
*
*******************************************************************************/

#include "CpuWidget.h"
#include "CpuWidget.moc"

#include "BaseContextMenu.h"
#include "CpuConfigurationDialog.h"
#include "IconSize.h"
#include "InformationDialog.h"
#include "ToolTipWidget.h"
#include "Util.h"
#include "XmlOptions.h"

#include <QApplication>
#include <QTextStream>
#include <algorithm>

namespace Cpu
{
    //__________________________________________________________
    CpuWidget::CpuWidget( QWidget *parent ):
        HistogramWidget( parent ),
        cpuMonitor_( this ),
        displayMask_( CpuLoad::All )
    {

        Debug::Throw( "Cpu::CpuWidget::CpuWidget.\n" );
        setWindowTitle( Util::windowTitle( tr( "Cpu Monitor" ) ) );

        colorMap_.insert( CpuLoad::User, Color( "USER_COLOR" ) );
        colorMap_.insert( CpuLoad::Nice, Color( "NICE_COLOR" ) );
        colorMap_.insert( CpuLoad::System, Color( "SYSTEM_COLOR" ) );

        _installActions();
        _configurationAction().setText( tr( "Cpu Monitor Settings..." ) );
        _updateConfiguration();

    }

    //______________________________________________________________________
    void CpuWidget::_installContextMenuActions( BaseContextMenu* menu )
    {
        Debug::Throw( "Cpu::CpuWidget::_installContextMenuActions.\n" );

        // create parent menu
        HistogramWidget::_installContextMenuActions( menu );

        // actions
        menu->insertAction( &_menuSeparator(), &_autoScaleAction() );

        // cpu menu
        menu->insertSeparator( &_menuSeparator() );
        foreach( QAction* action, displayActionGroup_->actions() )
        { menu->insertAction( &_menuSeparator(), action ); }

        menu->insertMenu( &_menuSeparator(), &cpuMonitor_.menu() );

    }

    //______________________________________________________________________
    bool CpuWidget::_updateToolTip( void )
    {

        Debug::Throw( "Cpu::CpuWidget::_updateToolTip.\n" );

        _toolTipWidget().setPixmap( windowIcon().pixmap( IconSize( IconSize::Maximum ) ) );

        // html formated tooltip
        // title
        QString buffer;
        QTextStream what( &buffer );
        what << "<div align='center' style='border: 10px;'><b>" << windowTitle() << "</b></div>" << endl;

        // header
        what << "<table style='margin: 10px;' >" << endl;
        what << "<tr><td align='right'>host: </td><td>" << _hostName() << "</td></tr>" << endl;
        what << "<tr><td align='right'>cpu: </td><td>" << cpuMonitor_.device() << "</td></tr>" << endl;
        what << endl;

        // data
        const CpuLoad::DataList& data( cpuMonitor_.data() );
        if( !data.empty() ) {
            what << "<tr><td align='right'>user: </td><td>" << data.back().ratio( CpuLoad::User ) << "%" << "</td></tr>" << endl;
            what << "<tr><td align='right'>system: </td><td>" << data.back().ratio( CpuLoad::System ) << "%" << "</td></tr>" << endl;
            what << "<tr><td align='right'>nice: </td><td>" << data.back().ratio( CpuLoad::Nice ) << "%" << "</td></tr>" << endl;
        } else what << "<tr><td colspan='2'>acquiring</td></tr>" << endl;
        what << "</table>" << endl;

        _toolTipWidget().setText( buffer );
        return _toolTipWidget().isHidden();

    }

    //______________________________________________________________________
    void CpuWidget::timerEvent( QTimerEvent* event )
    {

        if( event->timerId() == _updateTimer().timerId() )
        {

            cpuMonitor_.update();
            update();

        } else  HistogramWidget::timerEvent( event );

        return;

    }

    //______________________________________________________________________
    void CpuWidget::_paint( QPaintDevice& device, const QRect& clip )
    {

        // check border against window size
        if( !_margins().adjustedRect( rect() ).isValid() )
        {
            InformationDialog( this, tr( "Window size is too small with respect to margins.\n Margins set to 0." ) ).exec();
            _setMargins( 0 );
        }

        // store size
        QRect rect( _margins().adjustedRect( this->rect() ) );
        const int width( rect.width() );
        const int height( rect.height() );

        // create painter
        QPainter painter( &device );
        painter.setClipRect( clip );
        painter.setRenderHints(QPainter::Antialiasing);

        // retrieve data
        CpuLoad::DataList data( cpuMonitor_.data() );
        while( data.size() > width ) data.removeFirst();

        if( !data.empty() && displayMask_ )
        {

            const bool showUser( displayMask_ & CpuLoad::User );
            const bool showSystem( displayMask_ & CpuLoad::System );
            const bool showNice( displayMask_ & CpuLoad::Nice );

            // running points definition
            double xMin( _margins().left() + 1 );
            double x( xMin );

            double yMin( height + _margins().bottom() + 1 );
            double yMax( yMin );

            QPolygonF nicePolygon;
            QPolygonF systemPolygon;
            QPolygonF userPolygon;

            if( _autoScaleAction().isChecked() )
            {

                // autoscale painting
                double scale( _getScale( data ) );
                for( CpuLoad::DataList::const_iterator iter = data.begin(); iter != data.end(); ++iter )
                {

                    yMax = yMin;
                    if( showUser )
                    {
                        yMax -= qMax( 1.0, scale*iter->user()/iter->time() );
                        userPolygon << QPointF( x, yMax );
                    }

                    if( showSystem )
                    {
                        yMax -= qMax( 1.0, scale*iter->system()/iter->time() );
                        systemPolygon << QPointF( x, yMax );
                    }

                    if( showNice )
                    {
                        yMax -= qMax( 1.0, scale*iter->nice()/iter->time() );
                        nicePolygon << QPointF( x, yMax );
                    }

                    x+=1;
                }

            } else {

                for( CpuLoad::DataList::const_iterator iter = data.begin(); iter != data.end(); ++iter )
                {

                    yMax = yMin;

                    if( showUser )
                    {
                        yMax = yMin -  qMax( 1.0, double(height*iter->user())/iter->total() );
                        userPolygon << QPointF( x, yMax );
                    }

                    if( showSystem )
                    {
                        yMax -=  qMax( 1.0, double(height*iter->system())/iter->total() );
                        systemPolygon << QPointF( x, yMax );
                    }

                    if( showNice )
                    {
                        yMax -= qMax( 1.0, double(height*iter->nice())/iter->total() );
                        nicePolygon << QPointF( x, yMax );
                    }

                    x+=1;
                }

            }

            // close polygons
            /*
            polygons are manually 'subtracted' because it should work quicker than
            the general case Qt implementation
            */
            if( showNice )
            {
                if( showSystem )
                {

                    for( int i = systemPolygon.size()-1 ; i >= 0; i-- )
                    { nicePolygon << systemPolygon[i]; }

                } else if( showUser ) {

                    for( int i = userPolygon.size()-1; i >= 0; i-- )
                    { nicePolygon << userPolygon[i]; }

                } else {

                    nicePolygon << QPointF( x-1, yMin );
                    nicePolygon << QPointF( xMin, yMin );

                }

                nicePolygon << nicePolygon.front();
            }

            if( showSystem )
            {
                if( showUser ) {
                    for( int i = userPolygon.size()-1; i >= 0; i-- )
                    { systemPolygon << userPolygon[i]; }
                } else {
                    systemPolygon << QPointF( x-1, yMin );
                    systemPolygon << QPointF( xMin, yMin );
                }

                systemPolygon << systemPolygon.front();
            }

            if( showUser )
            {
                userPolygon << QPointF( x-1, yMin );
                userPolygon << QPointF( xMin, yMin );
                userPolygon << userPolygon.front();
            }

            // prepare pen
            QPen pen;
            pen.setJoinStyle( Qt::RoundJoin );
            pen.setCapStyle( Qt::RoundCap );
            pen.setWidthF( 1.5 );

            // draw polygons
            if( showNice )
            {

                pen.setColor( colorMap_[CpuLoad::Nice].foreground() );
                painter.setPen( pen );
                painter.setBrush( colorMap_[CpuLoad::Nice].background() );
                painter.drawPolygon( nicePolygon );

            }

            if( showSystem )
            {

                pen.setColor( colorMap_[CpuLoad::System].foreground() );
                painter.setPen( pen );
                painter.setBrush( colorMap_[CpuLoad::System].background() );
                painter.drawPolygon( systemPolygon );

            }

            if( showUser )
            {
                pen.setColor( colorMap_[CpuLoad::User].foreground() );
                painter.setPen( pen );
                painter.setBrush( colorMap_[CpuLoad::User].background() );
                painter.drawPolygon( userPolygon );
            }

        }

        // draw host
        if( _showLoadAction().isChecked() || data.empty() )
        { _paintLoad( painter, rect ); }

        // draw host
        _paintHost( painter, rect );

        painter.end();

    }

    //______________________________________________________________________
    void CpuWidget::resizeEvent( QResizeEvent* event )
    {
        int width = qMax( 0, event->size().width() - _margins().width() );
        cpuMonitor_.setDataSize( width );
        colorMap_[CpuLoad::User].updateGradient( event->size() );
        colorMap_[CpuLoad::System].updateGradient( event->size() );
        colorMap_[CpuLoad::Nice].updateGradient( event->size() );
        return HistogramWidget::resizeEvent( event );
    }

    //________________________________________________
    void CpuWidget::_configuration( void )
    {

        Debug::Throw( "Cpu::CpuWidget::_configuration.\n" );

        CpuConfigurationDialog dialog;
        _configureFromDialog( dialog );

    }

    //__________________________________________________________
    void CpuWidget::_updateConfiguration( void )
    {
        Debug::Throw( "Cpu::CpuWidget::_updateConfiguration.\n" );

        // colors
        colorMap_[CpuLoad::User].update();
        colorMap_[CpuLoad::Nice].update();
        colorMap_[CpuLoad::System].update();

        // update mask
        displayMask_ = 0;
        if( XmlOptions::get().get<bool>("SHOW_USER") ) displayMask_ |= CpuLoad::User;
        if( XmlOptions::get().get<bool>("SHOW_SYSTEM") ) displayMask_ |= CpuLoad::System;
        if( XmlOptions::get().get<bool>("SHOW_NICE") ) displayMask_ |= CpuLoad::Nice;
        _updateDisplayActions();

        _autoScaleAction().setChecked( XmlOptions::get().get<bool>( "AUTO_SCALE" ) );

        // base class update
        HistogramWidget::_updateConfiguration();
        update();

    }

    //________________________________________________
    void CpuWidget::_toggleAutoScale( bool state )
    {
        XmlOptions::get().set<bool>( "AUTO_SCALE", state );
        update();
    }

    //________________________________________________
    void CpuWidget::_toggleDisplay( QAction* action )
    {

        Debug::Throw( "Cpu::CpuWidget::_toggleDisplay.\n" );
        CpuLoad::DataType bit( displayActions_[action] );
        if( action->isChecked() ) displayMask_ |= bit;
        else displayMask_ &= ~bit;

        XmlOptions::get().set<bool>( "SHOW_USER", displayMask_ & CpuLoad::User );
        XmlOptions::get().set<bool>( "SHOW_SYSTEM", displayMask_ & CpuLoad::System );
        XmlOptions::get().set<bool>( "SHOW_NICE", displayMask_ & CpuLoad::Nice );

        update();

    }

    //______________________________________________________________________
    void CpuWidget::_installActions( void )
    {

        Debug::Throw( "Cpu::CpuWidget::_installActions.\n" );

        // display
        displayActionGroup_ = new QActionGroup( this );
        displayActionGroup_->setExclusive( false );
        displayActions_.insert( displayActionGroup_->addAction( tr( "User" ) ), CpuLoad::User );
        displayActions_.insert( displayActionGroup_->addAction( tr( "System" ) ), CpuLoad::System );
        displayActions_.insert( displayActionGroup_->addAction( tr( "Nice" ) ), CpuLoad::Nice );

        for( DisplayActionMap::iterator iter = displayActions_.begin(); iter != displayActions_.end(); ++iter )
        { iter.key()->setCheckable( true ); }

        _updateDisplayActions();
        connect( displayActionGroup_, SIGNAL(triggered(QAction*)), SLOT(_toggleDisplay(QAction*)) );

        // auto scale flags
        addAction( autoScaleAction_ = new QAction( tr( "Auto Scale" ), this ) );
        autoScaleAction_->setCheckable( true );
        connect( autoScaleAction_, SIGNAL(toggled(bool)), SLOT(_toggleAutoScale(bool)) );

    }

    //______________________________________________________________________
    void CpuWidget::_updateDisplayActions( void )
    {
        Debug::Throw( "Cpu::CpuWidget::_updateDisplayActions.\n" );
        for( DisplayActionMap::iterator iter = displayActions_.begin(); iter != displayActions_.end(); ++iter )
        {
            iter.key()->setChecked( displayMask_ & iter.value() );
            iter.key()->setIcon( colorMap_[ iter.value() ].icon() );
        }
        Debug::Throw( "Cpu::CpuWidget::_updateDisplayActions - done.\n" );
    }

    //_________________________________________________________
    double CpuWidget::_getScale( const CpuLoad::DataList& data ) const
    {
        Debug::Throw( "Cpu::CpuWidget::_getScale.\n" );

        Q_ASSERT( !data.empty() );

        // get maximum
        CpuLoad::DataList::const_iterator iter( std::max_element( data.begin(), data.end(), CpuLoad::LessFTor( displayMask_ ) ) );
        double maximum = double( iter->sum( displayMask_ ) )/iter->time();

        int height( qMax( 1, CpuWidget::height() - _margins().height() ) );
        return (maximum) ? double( height )/maximum:1;
    }

    //_________________________________________________________
    void CpuWidget::_paintLoad( QPainter& painter, QRect& rect )
    {
        Debug::Throw( "Cpu::CpuWidget::_paintLoad.\n" );

        // prepare load text
        QString buffer;
        const CpuLoad::DataList& data( cpuMonitor_.data() );
        if( !data.empty() ) buffer = QString( "%1: %2%" ).arg( cpuMonitor_.device() ).arg( cpuMonitor_.data().back().ratio() );
        else buffer  = QString( tr( "acquiring (%1)" ) ).arg( cpuMonitor_.device() );

        _paintText( painter, buffer, rect );
        return;

    }
}
