// $Id: DiskWidget.cpp,v 1.58 2014/02/10 10:07:45 hupereir Exp $

/******************************************************************************
*
* Copyright (C) 2002 Hugo PEREIRA <mailto: hugo.pereira@free.fr>
*
* This is free software; you can redistribute it and/or modify it under the
* terms of the GNU General Public License as published by the Free Software
* Foundation; either version 2 of the License, or (at your option) any later
* version.
*
* This software is distributed in the hope that it will be useful, but WITHOUT
* Any WARRANTY; without even the implied warranty of MERCHANTABILITY or
* FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
* for more details.
*
* You should have received a copy of the GNU General Public License along with
* this program.  If not, see <http://www.gnu.org/licenses/>.
*
*******************************************************************************/

#include "DiskWidget.h"
#include "DiskWidget.moc"

#include "BaseContextMenu.h"
#include "DiskConfigurationDialog.h"
#include "IconSize.h"
#include "InformationDialog.h"
#include "ToolTipWidget.h"
#include "Util.h"
#include "XmlOptions.h"

#include <QApplication>
#include <algorithm>

namespace Disk
{
    //__________________________________________________________
    DiskWidget::DiskWidget( QWidget *parent ):
        HistogramWidget( parent ),
        diskMonitor_( this ),
        displayMask_( Data::All )
    {

        Debug::Throw( "Disk::DiskWidget::DiskWidget.\n" );
        setWindowTitle( Util::windowTitle( "Disk Monitor" ) );

        colorMap_.insert( Data::Read, Color( "Read_COLOR" ) );
        colorMap_.insert( Data::Write, Color( "Write_COLOR" ) );

        _installActions();
        _configurationAction().setText( tr( "Disk Monitor Settings..." ) );
        _updateConfiguration();

    }

    //______________________________________________________________________
    void DiskWidget::_installContextMenuActions( BaseContextMenu* menu )
    {
        Debug::Throw( "Disk::DiskWidget::_installContextMenuActions.\n" );

        // create parent menu
        HistogramWidget::_installContextMenuActions( menu );

        // actions
        menu->insertSeparator( &_menuSeparator() );
        foreach( QAction* action, displayActionGroup_->actions() )
        { menu->insertAction( &_menuSeparator(), action ); }

        menu->insertMenu( &_menuSeparator(), &diskMonitor_.menu() );

    }

    //______________________________________________________________________
    bool DiskWidget::_updateToolTip( void )
    {

        Debug::Throw( "Disk::DiskWidget::_updateToolTip.\n" );

        _toolTipWidget().setPixmap( windowIcon().pixmap( IconSize( IconSize::Maximum ) ) );

        // html formated tooltip
        // title
        QString buffer;
        QTextStream what( &buffer );
        what << "<div align='center' style='border: 10px;'><b>" << windowTitle() << "</b></div>" << endl;

        // header
        what << "<table style='margin: 10px;' >" << endl;
        what << "<tr><td align='right'>" << tr( "Host" ) << ": </td><td>" << _hostName() << "</td></tr>" << endl;
        what << "<tr><td align='right'>" << tr( "Device" ) << ": </td><td>" << diskMonitor_.device() << "</td></tr>" << endl;
        what << endl;

        // data
        DataSamples::DataList data( diskMonitor_.data() );
        if( !data.empty() ) {
            what << "<tr><td align='right'>" << tr( "Reading" ) << ": </td><td>" << DiskLoad::toString( double(data.back().read())/data.back().time() ) << "</td></tr>" << endl;
            what << "<tr><td align='right'>" << tr( "Writing" ) << ": </td><td>" << DiskLoad::toString( double(data.back().write())/data.back().time() ) << "</td></tr>" << endl;
        } else what << "<tr><td colspan='2'>" << tr( "Acquiring" ) << "</td></tr>" << endl;
        what << "</table>"<< endl;

        _toolTipWidget().setText( buffer );

        return _toolTipWidget().isHidden();

    }

    //______________________________________________________________________
    void DiskWidget::timerEvent( QTimerEvent* event )
    {

        if( event->timerId() == _updateTimer().timerId() )
        {

            diskMonitor_.update();
            update();

        } else HistogramWidget::timerEvent( event );

        return;
    }

    //______________________________________________________________________
    void DiskWidget::_paint( QPaintDevice& device, const QRect& clip )
    {

        // check margins against window size
        if( !_margins().adjustedRect( rect() ).isValid() )
        {
            InformationDialog( this, tr( "Window size is too small with respect to margins.\n Margins set to 0." ) ).exec();
            _setMargins(0);
        }

        // store size
        QRect rect( _margins().adjustedRect( this->rect() ) );
        const int width( rect.width() );
        const int height( rect.height() );

        // create painter
        QPainter painter( &device );
        painter.setClipRect( clip );
        painter.setRenderHints(QPainter::Antialiasing);

        // retrieve data
        DataSamples::DataList data( diskMonitor_.data() );
        while( data.size() > width ) data.removeFirst();

        if( !data.empty() && displayMask_ )
        {

            const bool showWrite( displayMask_ & Data::Write );
            const bool showRead( displayMask_ & Data::Read );

            // running points definition
            double xMin( _margins().left() + 1 );
            double x( xMin );
            double yMin( height + _margins().bottom() + 1 );
            double yMax( yMin );

            QPolygonF writePolygon;
            QPolygonF readPolygon;

            // autoscale painting
            double scale( _getScale( data ) );
            for( DataSamples::DataList::const_iterator iter = data.begin(); iter != data.end(); ++iter )
            {

                yMax = yMin;

                if( showWrite )
                {
                    yMax -= qMax( 1.0, scale*iter->write()/iter->time() );
                    writePolygon << QPointF( x, yMax );
                }

                if( showRead )
                {
                    yMax -= qMax( 1.0, scale*iter->read()/iter->time() );
                    readPolygon << QPointF( x, yMax );
                }

                x+=1;
            }

            // close polygons
            /*
            polygons are manually 'subtracted' because it should work quicker than
            the general case Qt implementation
            */
            if( showRead )
            {
                if( showWrite ) {
                    for( int i = writePolygon.size()-1; i >= 0; i-- )
                    { readPolygon << writePolygon[i]; }
                } else {
                    readPolygon << QPointF( x-1, yMin );
                    readPolygon << QPointF( xMin, yMin );
                }

                readPolygon << readPolygon.front();
            }

            if( showWrite )
            {
                writePolygon << QPointF( x-1, yMin );
                writePolygon << QPointF( xMin, yMin );
                writePolygon << writePolygon.front();
            }

            // prepare pen
            QPen pen;
            pen.setJoinStyle( Qt::RoundJoin );
            pen.setWidthF( 1.5 );

            // draw read polygon
            if( showRead )
            {
                pen.setColor( colorMap_[Data::Read].foreground() );
                painter.setPen( pen );
                painter.setBrush( colorMap_[Data::Read].background() );
                painter.drawPolygon( readPolygon );

            }

            // draw write polygon
            if( showWrite )
            {
                pen.setColor( colorMap_[Data::Write].foreground() );
                painter.setPen( pen );
                painter.setBrush( colorMap_[Data::Write].background() );
                painter.drawPolygon( writePolygon );
            }

        }

        // draw load
        if( _showLoadAction().isChecked() || data.empty() )
        { _paintLoad( painter, rect ); }

        // draw host
        _paintHost( painter, rect );

        painter.end();

    }

    //______________________________________________________________________
    void DiskWidget::resizeEvent( QResizeEvent* event )
    {
        unsigned int width( qMax<unsigned int>( 0, event->size().width() - _margins().width() ) );
        diskMonitor_.setDataSize( width );
        colorMap_[Data::Read].updateGradient( event->size() );
        colorMap_[Data::Write].updateGradient( event->size() );
        return HistogramWidget::resizeEvent( event );
    }

    //________________________________________________
    void DiskWidget::_configuration( void )
    {

        Debug::Throw( "Disk::DiskWidget::_configuration.\n" );

        DiskConfigurationDialog dialog;
        _configureFromDialog( dialog );

    }

    //__________________________________________________________
    void DiskWidget::_updateConfiguration( void )
    {
        Debug::Throw( "Disk::DiskWidget::_updateConfiguration.\n" );

        // colors
        colorMap_[Data::Write].update();
        colorMap_[Data::Read].update();

        // update mask
        displayMask_ = 0;
        if( XmlOptions::get().get<bool>("SHOW_Read") ) displayMask_ |= Data::Read;
        if( XmlOptions::get().get<bool>("SHOW_Write") ) displayMask_ |= Data::Write;
        _updateDisplayActions();

        // base class update
        HistogramWidget::_updateConfiguration();
        update();
        Debug::Throw( "Disk::DiskWidget::_updateConfiguration - done.\n" );

    }

    //________________________________________________
    void DiskWidget::_toggleDisplay( QAction* action )
    {

        Debug::Throw( "Disk::DiskWidget::_toggleDisplay.\n" );
        Data::Type bit( displayActions_[action] );
        if( action->isChecked() ) displayMask_ |= bit;
        else displayMask_ &= ~bit;
        XmlOptions::get().set<bool>( "SHOW_Read", displayMask_ & Data::Read );
        XmlOptions::get().set<bool>( "SHOW_Write", displayMask_ & Data::Write );

        update();

    }

    //______________________________________________________________________
    void DiskWidget::_installActions( void )
    {

        Debug::Throw( "Disk::DiskWidget::_installActions.\n" );

        // display
        displayActionGroup_ = new QActionGroup( this );
        displayActionGroup_->setExclusive( false );
        displayActions_.insert( displayActionGroup_->addAction( tr( "Read" ) ), Data::Read );
        displayActions_.insert( displayActionGroup_->addAction( tr( "Write" ) ), Data::Write );

        for( DisplayActionMap::iterator iter = displayActions_.begin(); iter != displayActions_.end(); ++iter )
        { iter.key()->setCheckable( true ); }

        _updateDisplayActions();
        connect( displayActionGroup_, SIGNAL(triggered(QAction*)), SLOT(_toggleDisplay(QAction*)) );

    }

    //______________________________________________________________________
    void DiskWidget::_updateDisplayActions( void )
    {
        Debug::Throw( "Disk::DiskWidget::_updateDisplayActions.\n" );
        for( DisplayActionMap::iterator iter = displayActions_.begin(); iter != displayActions_.end(); ++iter )
        {
            iter.key()->setChecked( displayMask_ & iter.value() );
            iter.key()->setIcon( colorMap_[iter.value()].icon() );
        }
    }

    //_________________________________________________________
    double DiskWidget::_getScale( const DataSamples::DataList& data ) const
    {
        Debug::Throw( "Disk::DiskWidget::_getScale.\n" );

        Q_ASSERT( !data.empty() );

        // get maximum
        DataSamples::DataList::const_iterator iter( std::max_element( data.begin(), data.end(), DiskLoad::LessFTor( displayMask_ ) ) );
        double maximum = double( iter->sum( displayMask_ ) )/iter->time();

        unsigned int height( qMax<unsigned int>( 1, DiskWidget::height() - _margins().height() ) );
        return (maximum) ? double( height )/maximum:1;
    }

    //_________________________________________________________
    void DiskWidget::_paintLoad( QPainter& painter, QRect& rect )
    {
        Debug::Throw( "Disk::DiskWidget::_paintLoad.\n" );

        // prepare load text
        QString buffer;
        DataSamples::DataList data( diskMonitor_.data() );
        if( !data.empty() ) {

            buffer = QString( tr( "Writing: %1" ) ).arg( DiskLoad::toString( double(data.back().write())/data.back().time() ) );
            _paintText( painter, buffer, rect );

            buffer = QString( tr( "Reading: %1" ) ).arg( DiskLoad::toString( double(data.back().read())/data.back().time() ) );
            _paintText( painter, buffer, rect );

        } else {

            buffer = QString( tr( "Acquiring (%1)" ) ).arg( diskMonitor_.device()  );
            _paintText( painter, buffer, rect );

        }

        return;

    }
}
