// $Id: Application.cpp,v 1.67 2014/02/18 15:15:56 hupereir Exp $

/******************************************************************************
*
* Copyright (C) 2002 Hugo PEREIRA <mailto: hugo.pereira@free.fr>
*
* This is free software; you can redistribute it and/or modify it under the
* terms of the GNU General Public License as published by the Free Software
* Foundation; either version 2 of the License, or (at your option) any later
* version.
*
* This software is distributed in the hope that it will be useful, but WITHOUT
* Any WARRANTY; without even the implied warranty of MERCHANTABILITY or
* FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
* for more details.
*
* You should have received a copy of the GNU General Public License along with
* this program.  If not, see <http://www.gnu.org/licenses/>.
*
*******************************************************************************/

#include "Application.h"
#include "Application.moc"
#include "BaseIconNames.h"
#include "Debug.h"
#include "IconEngine.h"
#include "QtUtil.h"
#include "TransparentWindow.h"
#include "XmlOptions.h"

#include "Config.h"

#include <QApplication>
#include <QMessageBox>

//____________________________________________
Application::Application( CommandLineArguments arguments ) :
    BaseApplication( 0, arguments ),
    Counter( "Application" ),
    mainWindow_(0),
    type_( ApplicationType::AppNone )
{
    qApp->setQuitOnLastWindowClosed( false );
}

//____________________________________________
Application::~Application( void )
{ Debug::Throw( "Application::~Application.\n" ); }

//____________________________________________
bool Application::realizeWidget( void )
{
    Debug::Throw( "Application::realizeWidget.\n" );

    // check if the method has already been called.
    if( !BaseApplication::realizeWidget() ) return false;

    // rename about action
    aboutAction().setText( QString( tr( "About %1" ) ).arg( applicationName() ) );
    closeAction().setText( tr( "Close" ) );
    closeAction().setIcon( IconEngine::get( IconNames::DialogClose ) );
    closeAction().setShortcut( QKeySequence::Close );
    closeAction().setToolTip( tr( "Close this application" ) );

    // check top widget was set
    Q_ASSERT( hasMainWindow() );
    mainWindow().addAction( &closeAction() );
    connect( &mainWindow(), SIGNAL(geometryChanged(QRect)), SLOT(sendGeometry(QRect)) );
    connect( &mainWindow(), SIGNAL(optionChanged(XmlOption)), SLOT(sendOption(XmlOption)) );

    // configuration
    emit configurationChanged();

    mainWindow().show();
    return true;

}

//____________________________________________
void Application::usage( void ) const
{
    _usage( "transparency", tr( "[options] [applications]" ) );
    commandLineParser().usage();
    return;
}

//_________________________________________________
QString Application::applicationName( void ) const
{ return type_.name(); }

//_________________________________________________
QString Application::applicationVersion( void ) const
{ return VERSION; }

//_________________________________________________
QString Application::buildTimeStamp( void ) const
{ return BUILD_TIMESTAMP; }

    //@}

    //_________________________________________________
void Application::sendGeometry( QRect rect )
{
    Debug::Throw( "Application::sendGeometry.\n" );
    if( hasMainWindow() ) sendGeometry( rect, mainWindow().order() );
    else sendGeometry( rect, -1 );
}

//_________________________________________________
void Application::sendGeometry( QRect rect, int order )
{

    Debug::Throw( "Application::sendGeometry.\n" );
    if( !_hasApplicationManager() ) return;

    // send geometry
    Server::ApplicationId id( _applicationManager().id() );
    id.setName( "Transparency-settings" );

    // create command
    Server::ServerCommand command( id, Server::ServerCommand::Geometry );
    CommandLineArguments arguments;
    arguments
        << "transparency "
        << _applicationManager().id().name()
        << QString::number( rect.left() )
        << QString::number( rect.top())
        << QString::number( rect.width() )
        << QString::number( rect.height() )
        << QString::number( order )
        << type_.iconName();

    command.setArguments( arguments );

    // send to server
    _applicationManager().client().sendCommand( command );

}

//_________________________________________________
void Application::sendOption( XmlOption option )
{

    Debug::Throw( "Application::sendOption.\n" );
    if( !_hasApplicationManager() ) return;

    Debug::Throw()
        << "Application::sendOption -"
        << " name: " << option.name()
        << " value: " << option.raw()
        << endl;

    // create command
    Server::ApplicationId id( _applicationManager().id() );
    Server::ServerCommand command( id, Server::ServerCommand::Option );
    command.setXmlOption( option );

    _broadCast( command );

}

//_________________________________________________
void Application::sendRaiseRequest( void )
{

    Debug::Throw( "Application::sendRaiseRequest.\n" );
    _sendRequest( Server::ServerCommand::Raise );

}


//_________________________________________________
void Application::sendCloseRequest( void )
{

    Debug::Throw( "Application::sendRaiseRequest.\n" );
    _sendRequest( Server::ServerCommand::Abort );
    qApp->processEvents();
    closeAction().trigger();

}

//_________________________________________________
void Application::_sendRequest( Server::ServerCommand::CommandType command_type )
{

    Debug::Throw( "Application::_sendRequest.\n" );

    if( !_hasApplicationManager() ) return;

    // create command that match name
    Server::ApplicationId id( _applicationManager().id() );
    _broadCast( Server::ServerCommand( id, command_type ) );

}

//_________________________________________________
void Application::_broadCast( Server::ServerCommand command )
{

    Debug::Throw( "Application::_broadCast.\n" );

    if( !_hasApplicationManager() ) return;

    // loop over application types
    for( ApplicationType::Map::const_iterator iter = ApplicationType::names().begin(); iter != ApplicationType::names().end(); ++iter )
    {

        Debug::Throw() << "Application::_broadCast - sending to: " << iter.value() << endl;

        // create command that match name
        Server::ApplicationId id( _applicationManager().id() );
        id.setName( iter.value() );

        // create command
        command.setId( id );
        _applicationManager().client().sendCommand( command );

    }

}
//________________________________________________
bool Application::_processCommand( Server::ServerCommand command )
{

    // try base class
    if( BaseApplication::_processCommand( command ) ) return true;

    // test command type
    switch( command.command() )
    {
        case Server::ServerCommand::Raise:
        if( hasMainWindow() ) mainWindow().uniconify();
        return true;

        case Server::ServerCommand::GeometryRequest:
        if( hasMainWindow() ) sendGeometry( mainWindow().innerGeometry(), mainWindow().order() );
        return true;

        case Server::ServerCommand::Geometry:
        {
            // parse command
            CommandLineParser parser( commandLineParser( command.arguments() ) );
            QStringList arguments( parser.orphans() );
            if( arguments.size() >= 4 )
            {
                QRect rect(
                    QPoint( arguments[0].toInt(), arguments[1].toInt() ),
                    QSize( arguments[2].toInt(), arguments[3].toInt() ) );
                mainWindow().setInnerGeometry( rect );
            }

            if( arguments.size() >= 5 )
            { mainWindow().setOrder( arguments[4].toInt() ); }

            return true;
        }

        case Server::ServerCommand::Option:
        {
            mainWindow().processOption( command.option() );
            return true;
        }

        default: return false;

    }

}
