
// $Id: MainWindow.cpp,v 1.83 2014/02/10 10:34:30 hupereir Exp $

/******************************************************************************
*
* Copyright (C) 2002 Hugo PEREIRA <mailto: hugo.pereira@free.fr>
*
* This is free software; you can redistribute it and/or modify it under the
* terms of the GNU General Public License as published by the Free Software
* Foundation; either version 2 of the License, or (at your option) any later
* version.
*
* This software is distributed in the hope that it will be useful, but WITHOUT
* Any WARRANTY; without even the implied warranty of MERCHANTABILITY or
* FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
* for more details.
*
* You should have received a copy of the GNU General Public License along with
* this program.  If not, see <http://www.gnu.org/licenses/>.
*
*******************************************************************************/

#include "MainWindow.h"
#include "MainWindow.moc"

#include "Application.h"
#include "BaseContextMenu.h"
#include "IconEngine.h"
#include "IconNames.h"
#include "Singleton.h"
#include "ToolTipWidget.h"
#include "XmlOptions.h"

//_________________________________________________________________
MainWindow::MainWindow( QWidget* parent ):
    TransparentWindow( parent ),
    toolTipWidget_( 0 ),
    refresh_( 5 )
{
    Debug::Throw( "MainWindow::MainWindow.\n" );
    _installActions();
}

//_______________________________________________
bool MainWindow::event(QEvent *event)
{

    if (event->type() == QEvent::ToolTip && _updateToolTip() )
    {
        _toolTipWidget().adjustSize();
        _toolTipWidget().setIndexRect( QRect( static_cast<QHelpEvent*>(event)->globalPos(), QSize( 1, 1 ) ) );
        _toolTipWidget().show();
    }

    return TransparentWindow::event(event);

}

//_______________________________________________________
void MainWindow::timerEvent( QTimerEvent* event )
{

    // base class
    TransparentWindow::timerEvent( event );

    if( event->timerId() == _updateTimer().timerId() )
    {

        if( toolTipWidget_ && !toolTipWidget_->isHidden() )
        {
            Debug::Throw( "MainWindow::timerEvent - updating tooltip.\n" );
            _updateToolTip();
        }

    } else if( event->timerId() == _geometryTimer().timerId() ) {

        Debug::Throw( "MainWindow::timerEvent - saving geometry.\n" );

        // stop timer
        _geometryTimer().stop();

        // get window size and position
        const QRect geometry( innerGeometry() );
        const QSize size( geometry.size() );
        const QPoint position( _position() );

        // save size
        if( size.isValid() )
        {
            XmlOptions::get().set<int>( "WINDOW_WIDTH", size.width() );
            XmlOptions::get().set<int>( "WINDOW_HEIGHT", size.height() );
        }

        // save position
        XmlOptions::get().set<int>( "WINDOW_X", position.x() );
        XmlOptions::get().set<int>( "WINDOW_Y", position.y() );
        XmlOptions::write();

        // send geometry changed signal
        if( size.isValid() )
        { emit geometryChanged( geometry ); }

    }

    return;

}

//___________________________________________________
void MainWindow::mousePressEvent( QMouseEvent *event )
{
    if( _hasToolTipWidget() ) _toolTipWidget().hide();
    TransparentWindow::mousePressEvent( event );
}

//___________________________________________________
void MainWindow::mouseDoubleClickEvent( QMouseEvent *event )
{
    TransparentWindow::mouseDoubleClickEvent( event );
   if( event->button() == Qt::LeftButton ) _raiseAllAction().trigger();
    return;
}

//__________________________________________________________
ToolTipWidget& MainWindow::_toolTipWidget( void )
{
    if( !toolTipWidget_ )
    {
        toolTipWidget_ = new ToolTipWidget( this );
        toolTipWidget_->setFollowMouse( true );
        toolTipWidget_->setPreferredPosition( ToolTipWidget::Bottom );
    }
    return *toolTipWidget_;
}

//________________________________________________
void MainWindow::_installContextMenuActions( BaseContextMenu* menu )
{

    Debug::Throw( "MainWindow::_installContextMenuActions.\n" );
    TransparentWindow::_installContextMenuActions( menu );
    menu->addAction( closeAllAction_ );

}

//________________________________________________
void MainWindow::_updateConfiguration( void )
{

    Debug::Throw( "MainWindow::_updateConfiguration.\n" );

    TransparentWindow::_updateConfiguration();

    // outer padding
    const Base::Margins outerPadding( _outerPadding() );

    // size
    const QSize size(
        XmlOptions::get().get<int>( "WINDOW_WIDTH" ) + outerPadding.width(),
        XmlOptions::get().get<int>( "WINDOW_HEIGHT" ) + outerPadding.height() );
    if( size != MainWindow::size() ) resize( size );

    // position
    if( XmlOptions::get().contains( "WINDOW_X" ) && XmlOptions::get().contains( "WINDOW_Y" ) )
    { _setPosition( QPoint( XmlOptions::get().get<int>( "WINDOW_X" ), XmlOptions::get().get<int>( "WINDOW_Y" ) ) ); }

    // refresh
    refresh_ = XmlOptions::get().get<int>( "REFRESH_RATE" );

}

//________________________________________________
void MainWindow::_installActions( void )
{

    Debug::Throw( "MainWindow::_installActions.\n" );

    // raise all
    addAction( raiseAllAction_ = new QAction( IconEngine::get( IconNames::RaiseAll ), tr( "Raise All" ), this ) );
    raiseAllAction_->setToolTip( tr( "Raise all transparent windows" ) );
    connect( raiseAllAction_, SIGNAL(triggered()), Singleton::get().application(), SLOT(sendRaiseRequest()) );

    // close all
    addAction( closeAllAction_ = new QAction( IconEngine::get( IconNames::Exit ), tr( "Exit" ), this ) );
    closeAllAction_->setToolTip( tr( "Exit all transparent applications" ) );
    closeAllAction_->setShortcut( QKeySequence::Quit );
    connect( closeAllAction_, SIGNAL(triggered()), Singleton::get().application(), SLOT(sendCloseRequest()) );

}
