// $Id: TransparencyApplication.cpp,v 1.47 2014/02/18 15:15:56 hupereir Exp $

/******************************************************************************
*
* Copyright (C) 2002 Hugo PEREIRA <mailto: hugo.pereira@free.fr>
*
* This is free software; you can redistribute it and/or modify it under the
* terms of the GNU General Public License as published by the Free Software
* Foundation; either version 2 of the License, or (at your option) any later
* version.
*
* This software is distributed in the hope that it will be useful, but WITHOUT
* Any WARRANTY; without even the implied warranty of MERCHANTABILITY or
* FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
* for more details.
*
* You should have received a copy of the GNU General Public License along with
* this program.  If not, see <http://www.gnu.org/licenses/>.
*
*******************************************************************************/

#include "AcpiOptions.h"
#include "AcpiSystemOptions.h"
#include "CalendarOptions.h"
#include "CalendarXmlDef.h"
#include "ClockOptions.h"
#include "ClockXmlDef.h"
#include "CpuLoadOptions.h"
#include "DiskLoadOptions.h"
#include "HistogramOptions.h"
#include "MemoryLoadOptions.h"
#include "NetworkLoadOptions.h"
#include "SummaryOptions.h"
#include "TemperatureOptions.h"
#include "TemperatureSystemOptions.h"

#include "TransparencyApplication.h"
#include "TransparencyApplication.moc"

#include "AcpiWidget.h"
#include "CalendarWidget.h"
#include "ClockWidget.h"
#include "CpuWidget.h"
#include "DiskWidget.h"
#include "MemoryWidget.h"
#include "NetworkWidget.h"
#include "SummaryWidget.h"
#include "TemperatureWidget.h"

#include "XmlMigration.h"
#include "XmlOptions.h"

#include <QApplication>

//____________________________________________
CommandLineParser TransparencyApplication::commandLineParser( CommandLineArguments arguments, bool ignoreWarnings ) const
{
    Debug::Throw( "Application::commandLineParser.\n" );
    CommandLineParser out( Application::commandLineParser() );

    out.setGroup( CommandLineParser::applicationGroupName );
    out.registerFlag( "--acpi", tr( "transparent battery monitor" ) );
    out.registerFlag( "--calendar", tr( "transparent calendar" ) );
    out.registerFlag( "--clock", tr( "transparent clock" ) );
    out.registerFlag( "--cpu", tr( "transparent cpu monitor" ) );
    out.registerFlag( "--disk", tr( "transparent disk monitor" ) );
    out.registerFlag( "--memory", tr( "transparent memory monitor" ) );
    out.registerFlag( "--network", tr( "transparent network monitor" ) );
    out.registerFlag( "--temperature", tr( "transparent temperature monitor" ) );
    out.registerFlag( "--summary", tr( "text summary of all display" ) );
    out.registerFlag( "--all", tr( "all display except summary" ) );

    out.registerFlag( "--append", tr( "append selected applications to current session instead of replacing it" ) );

    if( !arguments.isEmpty() ) out.parse( arguments, ignoreWarnings );
    return out;

}

//____________________________________________
void TransparencyApplication::usage( void ) const
{
    _usage( "transparency", tr( "[options] [applications]" ) );
    commandLineParser().usage();
    return;
}

//____________________________________________
void TransparencyApplication::setApplicationType( const ApplicationType& type )
{
    Debug::Throw( "TransparencyApplication::setApplicationType.\n" );

    Application::setApplicationType( type );
    Debug::Throw() << "TransparencyApplication::setApplicationType - name: " << applicationName() << endl;

    switch( _type().type() )
    {

        case ApplicationType::AppAcpi:
        installAcpiOptions();
        installAcpiSystemOptions();
        XmlOptions::setFile( XmlOptions::get().raw( "RC_FILE" ) );
        XmlOptions::read();
        setMainWindow( new Acpi::AcpiWidget() );
        break;

        case ApplicationType::AppCalendar:
        installCalendarOptions();
        XmlOptions::setFile( XmlOptions::get().raw( "RC_FILE" ) );
        XmlOptions::read();
        XmlMigration( File(".transparent-calendar_db").addPath(Util::home() ), "DB_FILE", Calendar::Xml::TagList ).run();
        setMainWindow( new Calendar::CalendarWidget() );
        break;

        case ApplicationType::AppClock:
        installClockOptions();
        XmlOptions::setFile( XmlOptions::get().raw( "RC_FILE" ) );
        XmlOptions::read();
        XmlMigration( File(".transparent-clock_db").addPath(Util::home() ), "DB_FILE", Clock::Xml::AlarmList ).run();
        setMainWindow( new Clock::ClockWidget() );
        break;

        case ApplicationType::AppCpu:
        installHistogramOptions();
        installCpuLoadOptions();
        XmlOptions::setFile( XmlOptions::get().raw( "RC_FILE" ) );
        XmlOptions::read();
        setMainWindow( new Cpu::CpuWidget() );
        break;

        case ApplicationType::AppDisk:
        installHistogramOptions();
        installDiskLoadOptions();
        XmlOptions::setFile( XmlOptions::get().raw( "RC_FILE" ) );
        XmlOptions::read();
        setMainWindow( new Disk::DiskWidget() );
        break;

        case ApplicationType::AppMemory:
        installHistogramOptions();
        installMemoryLoadOptions();
        XmlOptions::setFile( XmlOptions::get().raw( "RC_FILE" ) );
        XmlOptions::read();
        setMainWindow( new Memory::MemoryWidget() );
        break;

        case ApplicationType::AppNetwork:
        installHistogramOptions();
        installNetworkLoadOptions();
        XmlOptions::setFile( XmlOptions::get().raw( "RC_FILE" ) );
        XmlOptions::read();
        setMainWindow( new Network::NetworkWidget() );
        break;

        case ApplicationType::AppSummary:
        installSummaryOptions();
        installAcpiSystemOptions();
        installTemperatureSystemOptions();
        XmlOptions::setFile( XmlOptions::get().raw( "RC_FILE" ) );
        XmlOptions::read();
        setMainWindow( new Summary::SummaryWidget() );
        break;

        case ApplicationType::AppTemperature:
        installHistogramOptions();
        installTemperatureOptions();
        installTemperatureSystemOptions();
        XmlOptions::setFile( XmlOptions::get().raw( "RC_FILE" ) );
        XmlOptions::read();
        setMainWindow( new Temperature::TemperatureWidget() );
        break;

        default:
        Debug::Throw(0) << "TransparencyApplication::setApplicationType - unsupported application type. exiting." << endl;
        qApp->quit();
        break;

    }

    // update top widget flag
    mainWindow().setAttribute( Qt::WA_AlwaysShowToolTips, true );

    // set debug level
    Debug::setLevel( XmlOptions::get().get<int>( "DEBUG_LEVEL" ) );
    if( Debug::level() ) Debug::Throw() << XmlOptions::get() << endl;

    Debug::Throw(0) << "transparency - running " << applicationName() << endl;

}
