#ifndef TransparentWindow_h
#define TransparentWindow_h

// $Id: TransparentWindow.h,v 1.39 2014/02/10 11:08:16 hupereir Exp $

/******************************************************************************
*
* Copyright (C) 2002 Hugo PEREIRA <mailto: hugo.pereira@free.fr>
*
* This is free software; you can redistribute it and/or modify it under the
* terms of the GNU General Public License as published by the Free Software
* Foundation; either version 2 of the License, or (at your option) any later
* version.
*
* This software is distributed in the hope that it will be useful, but WITHOUT
* Any WARRANTY; without even the implied warranty of MERCHANTABILITY or
* FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
* for more details.
*
* You should have received a copy of the GNU General Public License along with
* this program.  If not, see <http://www.gnu.org/licenses/>.
*
*******************************************************************************/

#include "Debug.h"
#include "TransparentWidget.h"
#include "XmlOption.h"
#include "XmlOptions.h"
#include "X11Defines.h"

#include <QBasicTimer>
#include <QTimerEvent>

#include <QAction>
#include <QMenu>
#include <QMoveEvent>
#include <QPaintEvent>
#include <QResizeEvent>

// forward declaration
class BaseContextMenu;
class ConfigurationDialog;

namespace Base { class HelpManager; }

//! transparent TransparentWindow widget
class TransparentWindow : public Transparency::TransparentWidget
{

    //! Qt meta object declaration
    Q_OBJECT

    public:

    //! constructor
    TransparentWindow( QWidget *parent = 0 );

    //! uniconify
    void uniconify( void );

    //! process option
    void processOption( const XmlOption& );

    //! order
    int order( void ) const
    { return order_; }

    //! order
    void setOrder( int value )
    {
        order_ = value;
        XmlOptions::get().set<bool>( "ORDER", value );
    }

    //! geometry
    QRect innerGeometry( void ) const;

    //! geometry
    void setInnerGeometry( const QRect& );

    Q_SIGNALS:

    //! emmited when a given option is changed that must be broadcasted
    void optionChanged( XmlOption );

    protected:

    //!@name actions
    //@{

    //! configure
    QAction& _configurationAction( void ) const
    { return *configurationAction_; }

    //! stay on top
    QAction& _staysOnTopAction( void ) const
    { return *staysOnTopAction_; }

    //! position is locked
    QAction& _lockPositionAction( void ) const
    { return *lockPositionAction_; }

    //! widget is hidden from taskBar
    QAction& _hideFromTaskBarAction( void ) const
    { return *hideFromTaskBarAction_; }

    //! widget is hidden from taskBar
    QAction& _stickyAction( void ) const
    { return *stickyAction_; }

    //! corners are rounded
    QAction& _roundCornersAction( void ) const
    { return *roundCornersAction_; }

    //! use svg
    QAction& _useSvgAction( void ) const
    { return *useSvgAction_; }

    //! reload background action
    QAction& _reloadBackgroundAction( void ) const
    { return *reloadBackgroundAction_; }

    //! use svg
    QAction& _lockFlagsAction( void ) const
    { return *lockFlagsAction_; }

    //! menu separator
    QAction& _menuSeparator( void ) const
    { return *menuSeparator_; }

    //@}

    //! install context menu actions
    virtual void _installContextMenuActions( BaseContextMenu* );

    //! configuration from dialog
    virtual void _configureFromDialog( ConfigurationDialog& dialog );

    //!@name event handlers
    //@{

    //! timer event
    virtual void timerEvent( QTimerEvent* );

    //! move event
    virtual void moveEvent( QMoveEvent* );

    //! resize event
    virtual void resizeEvent( QResizeEvent* );

    //! mouse press event [overloaded]
    virtual void mousePressEvent( QMouseEvent* );

    //! mouse press event [overloaded]
    virtual void mouseReleaseEvent( QMouseEvent *);

    //! mouse move event [overloaded]
    virtual void mouseMoveEvent( QMouseEvent* );

    //! double click event
    virtual void mouseDoubleClickEvent( QMouseEvent* );

    //! context menu event [overloaded]
    virtual void contextMenuEvent( QContextMenuEvent* );

    //@}

    //! geometry timer
    QBasicTimer& _geometryTimer( void )
    { return geometryTimer_; }

    //! current pressed button
    void _setButton( Qt::MouseButton button )
    { button_ = button; }

    //! current pressed button
    Qt::MouseButton _button( void ) const
    { return button_; }

    //! position
    void _setPosition( QPoint point );

    //! position
    QPoint _position( void ) const;

    //! handles foreground and fill color for histograms
    class Color: public Counter
    {

        public:

        //! constructor
        Color( const QString& name = QObject::tr( "Generic" ) ):
            Counter( "HistogramWidget::Color" ),
            name_( name ),
            foregroundGradient_( 0, 0, 1, 0 ),
            backgroundGradient_( 0, 0, 1, 0 ),
            useGradient_( false )
        {}

        //! update from options
        void update( void );

        //! update gradient
        void updateGradient( const QSize& size );

        //! foreground
        QColor foreground( void ) const
        { return foreground_; }

        //! background
        QBrush background( void ) const
        { return (useGradient_) ? QBrush( backgroundGradient_ ) : QBrush( background_ ); }

        //! use gradient
        bool useGradient( void ) const
        { return useGradient_; }

        //! gradient
        QLinearGradient foregroundGradient( void ) const
        { return foregroundGradient_; }

        //! gradient
        QLinearGradient backgroundGradient( void ) const
        { return backgroundGradient_; }

        //! icon (used to display in menus)
        QIcon icon( void ) const;

        private:

        //! name
        QString name_;

        //! foreground color
        QColor foreground_;

        //! background color
        QColor background_;

        //! foreground gradient
        QLinearGradient foregroundGradient_;

        //! background gradient
        QLinearGradient backgroundGradient_;

        //! brush
        bool useGradient_;

    };

    protected Q_SLOTS:

    //! configuration
    virtual void _configuration( void );

    //! update configuration
    virtual void _updateConfiguration( void );

    //! stays on top
    virtual void _toggleStaysOnTop( bool );

    //! lock position
    virtual void _toggleLockPosition( bool );

    //! toggle hide from taskBar
    virtual void _toggleHideFromTaskBar( bool );

    //! toggle window stickyness
    virtual void _toggleSticky( bool );

    //! round corners
    virtual void _toggleRoundCorners( bool );

    //! svg
    virtual void _toggleSvg( bool );

    //! toggle inverse colors
    virtual void _toggleInverseColors( bool );

    //! lock flags
    virtual void _toggleLockFlags( bool );

    //! background pixmap
    virtual void _updateBackgroundPixmap( void );

    //! update position based on desktop resized event
    virtual void _desktopResized( void );

    private:

    //! install actions
    void _installActions( void );

    //! anchor
    const unsigned int& _anchor( void ) const
    { return anchor_; }

    //! widget mask
    void _updateMask( void );

    //! returns closest point on grid, matching argument
    QPoint _gridPoint( QPoint ) const;

    //! get corner from (relative) position
    X11Defines::Direction _direction( const QPoint& ) const;

    //!@name actions
    //@{

    //! configuration action
    QAction* configurationAction_;

    //! stay on top
    QAction* staysOnTopAction_;

    //! position is locked
    QAction* lockPositionAction_;

    //! hide from taskBar
    QAction* hideFromTaskBarAction_;

    //! make window sticky
    QAction* stickyAction_;

    //! corners are rounded
    QAction* roundCornersAction_;

    //! use svg for display
    QAction* useSvgAction_;

    //! lock flags
    QAction* lockFlagsAction_;

    //! reload background
    QAction* reloadBackgroundAction_;

    //! menu separator
    /*! used by daughter classes to insert actions in the context menu */
    QAction* menuSeparator_;

    //@}

    //! position
    QPoint position_;

    //! click position
    QPoint clickPosition_;

    //! size at click time
    QSize clickSize_;

    //! window resize direction
    X11Defines::Direction direction_;

    //! geometry timer
    /*! used to save widget size and position after move/resize */
    QBasicTimer geometryTimer_;

    //! desktop resize timer
    QBasicTimer desktopResizeTimer_;

    //! button state
    Qt::MouseButton button_;

    //! anchor
    unsigned int anchor_;

    //! order (used to layout multiple transparent applications)
    int order_;

    //! movement grid
    int grid_;

    //! help manager
    Base::HelpManager *help_;

};


#endif
