// $Id: MemoryWidget.cpp,v 1.54 2014/02/10 10:07:45 hupereir Exp $

/******************************************************************************
*
* Copyright (C) 2002 Hugo PEREIRA <mailto: hugo.pereira@free.fr>
*
* This is free software; you can redistribute it and/or modify it under the
* terms of the GNU General Public License as published by the Free Software
* Foundation; either version 2 of the License, or (at your option) any later
* version.
*
* This software is distributed in the hope that it will be useful, but WITHOUT
* Any WARRANTY; without even the implied warranty of MERCHANTABILITY or
* FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
* for more details.
*
* You should have received a copy of the GNU General Public License along with
* this program.  If not, see <http://www.gnu.org/licenses/>.
*
*******************************************************************************/

#include "InformationDialog.h"

#include "MemoryWidget.h"
#include "MemoryWidget.moc"

#include "BaseContextMenu.h"
#include "IconSize.h"
#include "InformationDialog.h"
#include "KernelVersion.h"
#include "MemoryConfigurationDialog.h"
#include "ToolTipWidget.h"
#include "Util.h"
#include "XmlOptions.h"

#include <QApplication>
#include <QTextStream>
#include <algorithm>

namespace Memory
{

    //__________________________________________________________
    MemoryWidget::MemoryWidget( QWidget *parent ):
        HistogramWidget( parent ),
        displayMask_( MemoryLoad::All )
    {

        Debug::Throw( "MemoryWidget::MemoryWidget.\n" );
        setWindowTitle( Util::windowTitle( tr( "Memory Monitor" ) ) );

        colorMap_.insert( MemoryLoad::Memory, Color( "MEMORY_COLOR" ) );
        colorMap_.insert( MemoryLoad::Swap, Color( "SWAP_COLOR" ) );

        _installActions();
        _configurationAction().setText( tr( "Memory Monitor Settings..." ) );
        _updateConfiguration();

    }

    //______________________________________________________________________
    void MemoryWidget::_installContextMenuActions( BaseContextMenu* menu )
    {
        Debug::Throw( "SummaryWidget::_installContextMenuActions.\n" );

        // create parent menu
        HistogramWidget::_installContextMenuActions( menu );

        // actions
        menu->insertAction( &_menuSeparator(), autoScaleAction_ );

        // memory menu
        menu->insertSeparator( &_menuSeparator() );
        foreach( QAction* action, displayActionGroup_->actions() )
        { menu->insertAction( &_menuSeparator(), action ); }

    }

    //______________________________________________________________________
    bool MemoryWidget::_updateToolTip( void )
    {

        Debug::Throw( "MemoryWidget::_updateToolTip.\n" );

        _toolTipWidget().setPixmap( windowIcon().pixmap( IconSize( IconSize::Maximum ) ) );

        // html formated tooltip
        // title
        QString buffer;
        QTextStream what( &buffer );
        what << "<div align='center' style='border: 10px;'><b>" << windowTitle() << "</b></div>" << endl;

        // header
        what << "<table style='margin: 10px;' >" << endl;
        what << "<tr><td align='right'>host: </td><td>" << _hostName() << "</td></tr>" << endl;
        what << endl;

        // data
        const MemoryLoad::DataList& data( memoryLoad_.data() );
        if( !data.empty() ) {
            what << "<tr><td align='right'>" << tr( "Memory: " ) << "</td><td>" << data.back().memoryRatio() << "%" << "</td></tr>" << endl;
            what << "<tr><td align='right'>" << tr( "Swap: " ) << "</td><td>" << data.back().swapRatio() << "%" << "</td></tr>" << endl;
        } else what << "<tr><td colspan='2'>" << tr( "Acquiring" ) << "</td></tr>" << endl;
        what << "</table>"<< endl;

        _toolTipWidget().setText( buffer );
        return _toolTipWidget().isHidden();
    }

    //______________________________________________________________________
    void MemoryWidget::timerEvent( QTimerEvent* event )
    {

        if( event->timerId() == _updateTimer().timerId() )
        {
            memoryLoad_.update();
            update();
        }  HistogramWidget::timerEvent( event );

        return;

    }

    //______________________________________________________________________
    void MemoryWidget::_paint( QPaintDevice& device, const QRect& clip )
    {

        // check border against window size
        if( !_margins().adjustedRect( rect() ).isValid() )
        {
            InformationDialog( this, tr( "Window size is too small with respect to margins.\n Margins set to 0." ) ).exec();
            _setMargins(0);
        }

        // store size
        QRect rect( _margins().adjustedRect( this->rect() ) );
        const int width( rect.width() );
        const int height( rect.height() );

        // create painter
        QPainter painter( &device );
        painter.setClipRect( clip );
        painter.setRenderHints(QPainter::Antialiasing);

        // retrieve data
        MemoryLoad::DataList data( memoryLoad_.data() );
        while( data.size() > width ) data.removeFirst();

        // store flags locally to spare time
        if( !data.empty() && displayMask_ )
        {

            const bool showMemory( displayMask_ & MemoryLoad::Memory );
            const bool showSwap( displayMask_ & MemoryLoad::Swap );

            // running points definition
            double xMin( _margins().left() + 1 );
            double x( xMin );
            double yMin( height + _margins().bottom() + 1 );
            double yMax( yMin );

            QPolygonF swap_polygon;
            QPolygonF memory_polygon;

            if( _autoScaleAction().isChecked() )
            {

                // autoscale painting
                double scale( _getScale( data ) );
                for( MemoryLoad::DataList::const_iterator iter = data.begin(); iter != data.end(); ++iter )
                {

                    yMax = yMin;
                    if( showMemory )
                    {
                        yMax -= qMax( 1.0, scale*iter->memory() );
                        memory_polygon << QPointF( x, yMax );
                    }

                    if( showSwap )
                    {
                        yMax -= qMax( 1.0, scale*iter->swap() );
                        swap_polygon << QPointF( x, yMax );
                    }

                    x+=1;
                }

            } else {

                for( MemoryLoad::DataList::const_iterator iter = data.begin(); iter != data.end(); ++iter )
                {

                    yMax = yMin;
                    unsigned long total( iter->total( displayMask_ ) );
                    if( showMemory )
                    {
                        yMax = yMin -  qMax( 1.0, double(height*iter->memory())/total );
                        memory_polygon << QPointF( x, yMax );
                    }

                    if( showSwap )
                    {
                        yMax -=  qMax( 1.0, double(height*iter->swap())/total );
                        swap_polygon << QPointF( x, yMax );
                    }

                    x+=1;
                }

            }

            // close polygons
            /*
            polygons are manually 'subtracted' because it should work quicker than
            the general case Qt implementation
            */
            if( showSwap )
            {
                if( showMemory ) {
                    for( int i = memory_polygon.size()-1; i >= 0; i-- )
                    { swap_polygon << memory_polygon[i]; }
                } else {
                    swap_polygon << QPointF( x-1, yMin );
                    swap_polygon << QPointF( xMin, yMin );
                }

                swap_polygon << swap_polygon.front();
            }

            if( showMemory )
            {
                memory_polygon << QPointF( x-1, yMin );
                memory_polygon << QPointF( xMin, yMin );
                memory_polygon << memory_polygon.front();
            }

            // prepare pen
            QPen pen;
            pen.setJoinStyle( Qt::RoundJoin );
            pen.setWidthF( 1.5 );

            // draw swap polygon
            if( showSwap )
            {
                pen.setColor( colorMap_[MemoryLoad::Swap].foreground() );
                painter.setPen( pen );
                painter.setBrush( colorMap_[MemoryLoad::Swap].background() );
                painter.drawPolygon( swap_polygon );
            }

            // draw memory polygon
            if( showMemory )
            {
                pen.setColor( colorMap_[MemoryLoad::Memory].foreground() );
                painter.setPen( pen );
                painter.setBrush( colorMap_[MemoryLoad::Memory].background() );
                painter.drawPolygon( memory_polygon );
            }

        }

        // draw load
        if( _showLoadAction().isChecked() || data.empty() )
        { _paintLoad( painter, rect ); }

        // draw host
        _paintHost( painter, rect );

        painter.end();

    }

    //______________________________________________________________________
    void MemoryWidget::resizeEvent( QResizeEvent* event )
    {
        unsigned int width = qMax<unsigned int>( 0, event->size().width() - _margins().width() );
        memoryLoad_.setDataSize( width );
        colorMap_[MemoryLoad::Memory].updateGradient( event->size() );
        colorMap_[MemoryLoad::Swap].updateGradient( event->size() );
        return HistogramWidget::resizeEvent( event );
    }

    //________________________________________________
    void MemoryWidget::_configuration( void )
    {

        Debug::Throw( "MemoryWidget::_configuration.\n" );

        MemoryConfigurationDialog dialog;
        _configureFromDialog( dialog );

    }

    //__________________________________________________________
    void MemoryWidget::_updateConfiguration( void )
    {
        Debug::Throw( "MemoryWidget::_updateConfiguration.\n" );

        // colors
        colorMap_[MemoryLoad::Memory].update();
        colorMap_[MemoryLoad::Swap].update();

        // samples
        size_t samples( XmlOptions::get().get<size_t>( "SAMPLES" ) );
        memoryLoad_.setSampleSize( samples );

        // update mask
        displayMask_ = 0;
        if( XmlOptions::get().get<bool>("SHOW_MEMORY") ) displayMask_ |= MemoryLoad::Memory;
        if( XmlOptions::get().get<bool>("SHOW_SWAP") ) displayMask_ |= MemoryLoad::Swap;
        _updateDisplayActions();

        _autoScaleAction().setChecked( XmlOptions::get().get<bool>( "AUTO_SCALE" ) );

        // kernel version from options
        memoryLoad_.setKernelVersion( (Main::KernelVersion) XmlOptions::get().get<int>( "KERNEL_VERSION" ) );

        // base class update
        HistogramWidget::_updateConfiguration();
        update();
        Debug::Throw( "MemoryWidget::_updateConfiguration - done.\n" );

    }

    //________________________________________________
    void MemoryWidget::_toggleAutoScale( bool state )
    {
        XmlOptions::get().set<bool>( "AUTO_SCALE", state );
        update();
    }

    //________________________________________________
    void MemoryWidget::_toggleDisplay( QAction* action )
    {

        Debug::Throw( "MemoryWidget::_toggleDisplay.\n" );
        MemoryLoad::DataType bit( displayActions_[action] );
        if( action->isChecked() ) displayMask_ |= bit;
        else displayMask_ &= ~bit;

        XmlOptions::get().set<bool>( "SHOW_MEMORY", displayMask_ & MemoryLoad::Memory );
        XmlOptions::get().set<bool>( "SHOW_SWAP", displayMask_ & MemoryLoad::Swap );

        update();

    }

    //______________________________________________________________________
    void MemoryWidget::_installActions( void )
    {

        Debug::Throw( "MemoryWidget::_installActions.\n" );

        // display
        displayActionGroup_ = new QActionGroup( this );
        displayActionGroup_->setExclusive( false );
        displayActions_.insert( displayActionGroup_->addAction( tr( "User" ) ), MemoryLoad::Memory );
        displayActions_.insert( displayActionGroup_->addAction( tr( "System" ) ), MemoryLoad::Swap );

        for( DisplayActionMap::iterator iter = displayActions_.begin(); iter != displayActions_.end(); ++iter )
        { iter.key()->setCheckable( true ); }

        _updateDisplayActions();
        connect( displayActionGroup_, SIGNAL(triggered(QAction*)), SLOT(_toggleDisplay(QAction*)) );

        addAction( autoScaleAction_ = new QAction( tr( "Auto Scale" ), this ) );
        autoScaleAction_->setCheckable( true );
        connect( autoScaleAction_, SIGNAL(toggled(bool)), SLOT(_toggleAutoScale(bool)) );

    }

    //______________________________________________________________________
    void MemoryWidget::_updateDisplayActions( void )
    {
        Debug::Throw( "MemoryWidget::_updateDisplayActions.\n" );
        for( DisplayActionMap::iterator iter = displayActions_.begin(); iter != displayActions_.end(); ++iter )
        {
            iter.key()->setChecked( displayMask_ & iter.value() );
            iter.key()->setIcon( colorMap_[iter.value()].icon() );
        }
    }

    //_________________________________________________________
    double MemoryWidget::_getScale( const MemoryLoad::DataList& data ) const
    {
        Debug::Throw( "MemoryWidget::_getScale.\n" );

        Q_ASSERT( !data.empty() );

        // get maximum
        MemoryLoad::DataList::const_iterator iter( std::max_element( data.begin(), data.end(), MemoryLoad::LessFTor( displayMask_ ) ) );
        double maximum( iter->sum( displayMask_ ));

        unsigned int height( qMax<unsigned int>( 1, MemoryWidget::height() - _margins().height() ) );
        return (maximum) ? double( height )/maximum:1;
    }

    //_________________________________________________________
    void MemoryWidget::_paintLoad( QPainter& painter, QRect& rect )
    {
        Debug::Throw( "MemoryWidget::_paintLoad.\n" );


        // prepare load text
        QString buffer;
        const MemoryLoad::DataList& data( memoryLoad_.data() );
        if( !data.empty() )
        {

            buffer += QString( tr( "Memory: %1% Swap: %2%" ) )
                .arg( memoryLoad_.data().back().memoryRatio() )
                .arg( memoryLoad_.data().back().swapRatio() );

        } else buffer = QString( tr( "Acquiring (memory)" ) );

        _paintText( painter, buffer, rect );
        return;

    }

}
