#ifndef NetworkLoad_h
#define NetworkLoad_h

// $Id: NetworkLoad.h,v 1.18 2014/02/18 15:15:56 hupereir Exp $

/******************************************************************************
*
* Copyright (C) 2002 Hugo PEREIRA <mailto: hugo.pereira@free.fr>
*
* This is free software; you can redistribute it and/or modify it under the
* terms of the GNU General Public License as published by the Free Software
* Foundation; either version 2 of the License, or (at your option) any later
* version.
*
* This software is distributed in the hope that it will be useful, but WITHOUT
* Any WARRANTY; without even the implied warranty of MERCHANTABILITY or
* FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
* for more details.
*
* You should have received a copy of the GNU General Public License along with
* this program.  If not, see <http://www.gnu.org/licenses/>.
*
*******************************************************************************/


#include "Counter.h"

#include <QDateTime>
#include <QString>
#include <QTextStream>
#include <QHash>
#include <QList>
#include <QSet>


namespace Network
{

    //! measure relative network load from /proc/net/dev file
    class NetworkLoad: public Counter
    {

        public:

        //! data type
        enum DataType
        {
            Received = 1<<0,
            Transmitted = 1<<1,
            All = Received|Transmitted
        };

        Q_DECLARE_FLAGS( DataTypes, DataType )

        //! constructor
        NetworkLoad( void );

    //! convert number into string with proper unit
    static QString toString( double value );

    //! network load data
    class Data
    {

        public:

        //! constructor
        Data( void ):
            bytes_recieved_( 0 ),
            bytes_transmitted_( 0 ),
            packets_recieved_( 0 ),
            packets_transmitted_( 0 ),
            valid_( false )
        {}

        //! difference between two points
        Data operator - ( const Data& data ) const
        {
            Data out( *this );
            out.bytes_recieved_ = bytes_recieved_ - data.bytes_recieved_;
            out.bytes_transmitted_ = bytes_transmitted_ - data.bytes_transmitted_;
            out.packets_recieved_ = packets_recieved_ - data.packets_recieved_;
            out.packets_transmitted_ = packets_transmitted_ - data.packets_transmitted_;
            out.time_ = time_ - data.time_;
            out.valid_ = ( valid_ && data.valid_ && out.time_ > 0 );
            return out;
        }

        //! load from stream
        friend QTextStream &operator >> ( QTextStream &in, Data& data )
        {

            long long int errs( 0 ), drop( 0 ), fifo( 0 ), frame( 0 ), compressed( 0 ), multicast( 0 );
            in
                >> data.bytes_recieved_ >> data.packets_recieved_
                >> errs >> drop >> fifo >> frame >> compressed >> multicast
                >> data.bytes_transmitted_ >> data.packets_transmitted_;

            data.valid_ = in.status() == QTextStream::Ok;
            return in;
        }

        //! get recieved bytes
        long long int bytesRecieved( void ) const
        { return bytes_recieved_; }

        //! get transmitted bytes
        long long int bytesTransmitted( void ) const
        { return bytes_transmitted_; }

        //! get recieved packets
        long long int packetsRecieved( void ) const
        { return packets_recieved_; }

        //! get transmitted packets
        long long int packetsTransmitted( void ) const
        { return packets_transmitted_; }

        //! sum
        long long int sum( const int &type ) const
        {
            long long int out( 0 );
            if( type & Transmitted ) out += bytesTransmitted();
            if( type & Received ) out += bytesRecieved();
            return out;
        }

        //! set time
        void setTime( int time )
        { time_ = time; }

        //! get time
        int time( void ) const
        { return time_; }

        //! get validity
        bool valid( void ) const
        { return valid_; }

        private:

        //! recieved bytes
        long long int bytes_recieved_;

        //! transmitted bytes
        long long int bytes_transmitted_;

        //! recieved packets
        long long int packets_recieved_;

        //! transmitted packets
        long long int packets_transmitted_;

        //! measurement time
        int time_;

        //! true if current data are valid
        bool valid_;

    };

    //! shortcut for data list
    typedef QList< Data > DataList;

    //! data samples storage
    class DataSamples
    {

        public:

        //! construtor
        DataSamples( int sampleSize = 0, int dataSize = 0 ):
            sampleSize_( sampleSize ),
            dataSize_( dataSize )
        { }

        //! destructor
        ~DataSamples( void )
        {}

        //! set sample size
        void setSampleSize( int sampleSize );

        //! set data point size
        void setDataSize( int dataSize );

        //! retrieve samples
        const DataList& samples( void ) const
        { return sampleList_; }

        //! retrieve data
        const DataList& data( void ) const
        { return dataList_; }

        //! add a data point
        void add( const Data& data );

        private:

        //! list of measurement
        DataList sampleList_;

        //! list of differences
        DataList dataList_;

        //! number of samples to be averaged to make the data points
        int sampleSize_;

        //! number of data point to store
        int dataSize_;

    };

    //! shortcut for DataSamples set
    typedef QHash< QString, DataSamples > SampleMap;

    //! shortcut for Device set
    typedef QSet< QString > DeviceSet;

    //! get available devices names
    DeviceSet devices( void ) const;

    //! connected devices
    DeviceSet connectedDevices( void ) const;

    //! changes sampleSize_
    void setSampleSize( int value )
    {
        sampleSize_ = value;
        for( SampleMap::iterator iter = sampleMap_.begin(); iter != sampleMap_.end(); iter++ )
        { iter.value().setSampleSize( value ); }
    }

    //! changes sampleSize_
    void setDataSize( int value )
    {
        dataSize_ = value;
        for( SampleMap::iterator iter = sampleMap_.begin(); iter != sampleMap_.end(); iter++ )
        { iter.value().setDataSize( value ); }
    }

    //! retrieve data for a given device  name
    DataList data( const QString& device_name = "eth0" ) const;

    //! update all dataSamples from file
    void update( void );


    //! used to sort data according to selected data type
    class LessFTor
    {

        public:

        //! constructor
        LessFTor( const int& type ):
            type_( type )
            {}

            //! predicate
            bool operator()( const Data& first, const Data& second ) const
            { return double(first.sum( type_ ))/first.time() < double( second.sum( type_ ))/second.time(); }

            private:

            //! data type
            int type_;

        };

        private:

        //! filename
        const QString filename_;

        //! routes filename
        const QString routeFilename_;

        class Route
        {
            public:

            //! device
            QString device_;

            //! network
            QString network_;

            //! gateway
            QString gateway_;

            //! metric
            int metric_;

            //! validity
            bool valid_;

            // streamer
            friend QTextStream& operator >> ( QTextStream& in, Route& route )
            {
                int dummy;
                in >> route.device_ >> route.network_ >> route.gateway_ >> dummy >> dummy >> dummy >> route.metric_;
                route.valid_ = (in.status() == QTextStream::Ok );
                return in;
            }
        };

        //! number of samples to average on
        int sampleSize_;

        //! number of data to be kept for each device
        int dataSize_;

        //! data samples
        SampleMap sampleMap_;

        //! timer
        QTime timer_;

    };

    Q_DECLARE_OPERATORS_FOR_FLAGS( NetworkLoad::DataTypes )

}

#endif
