// $Id: NetworkMonitor.cpp,v 1.7 2014/02/10 10:07:45 hupereir Exp $

/******************************************************************************
*
* Copyright (C) 2002 Hugo PEREIRA <mailto: hugo.pereira@free.fr>
*
* This is free software; you can redistribute it and/or modify it under the
* terms of the GNU General Public License as published by the Free Software
* Foundation; either version 2 of the License, or (at your option) any later
* version.
*
* This software is distributed in the hope that it will be useful, but WITHOUT
* Any WARRANTY; without even the implied warranty of MERCHANTABILITY or
* FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
* for more details.
*
* You should have received a copy of the GNU General Public License along with
* this program.  If not, see <http://www.gnu.org/licenses/>.
*
*******************************************************************************/

#include "NetworkMonitor.h"
#include "NetworkMonitor.moc"

#include "Application.h"
#include "Debug.h"
#include "Singleton.h"
#include "SystemNotifications.h"
#include "XmlOptions.h"

namespace Network
{
    //__________________________________________________________
    NetworkMonitor::NetworkMonitor( QObject *parent ):
        QObject( parent ),
        Counter( "Network::NetworkMonitor" ),
        networkMenu_( 0 ),
        systemNotifications_( 0 )
    {

        Debug::Throw( "Network::NetworkMonitor::NetworkMonitor.\n" );

        networkMenu_ = new NetworkMenu();
        connect( networkMenu_, SIGNAL(deviceSelected(QString)), SLOT(_selectDeviceFromMenu(QString)) );
        connect( networkMenu_, SIGNAL(automaticDeviceSelected()), SLOT(_selectAutomaticDevice()) );

        connect( Singleton::get().application(), SIGNAL(configurationChanged()), SLOT(_updateConfiguration()) );
        _updateConfiguration();

    }

    //______________________________________________________________________
    void NetworkMonitor::timerEvent( QTimerEvent* event )
    {

        if( event->timerId() == deviceTimer_.timerId() )  _checkDevice();
        else return QObject::timerEvent( event );

    }

    //________________________________________________
    void NetworkMonitor::_selectDeviceFromMenu( QString device )
    {
        Debug::Throw( "Network::NetworkMonitor::_selectDeviceFromMenu.\n" );
        if( deviceTimer_.isActive() ) deviceTimer_.stop();
        XmlOptions::get().set<bool>( "NETWORK_AUTO_SELECT", false );
        _selectDevice( device );
    }

    //________________________________________________
    void NetworkMonitor::_selectAutomaticDevice( void )
    {
        Debug::Throw( "Network::NetworkMonitor::_selectAutomaticDevice.\n" );

        // check current device
        _checkDevice();
        XmlOptions::get().set<bool>( "NETWORK_AUTO_SELECT", true );

        // restart timer
        if( !deviceTimer_.isActive() ) deviceTimer_.start( 1000*XmlOptions::get().get<int>( "NETWORK_AUTO_SELECT_RATE" ), this );

    }

    //________________________________________________
    void NetworkMonitor::_checkDevice( QString device )
    {

        Debug::Throw( "Network::NetworkMonitor::_checkDevice.\n" );

        // get connected devices
        NetworkLoad::DeviceSet devices( networkLoad_.connectedDevices() );

        // check if found
        if( device.isEmpty() ) device = device_;
        if( devices.find( device ) == devices.end() && !devices.empty() )
        { device = *devices.begin(); }

        // select
        _selectDevice( device );

    }

    //________________________________________________
    void NetworkMonitor::_selectDevice( QString device )
    {

        Debug::Throw( "Network::NetworkMonitor::_selectDevice.\n" );
        if( device_ == device ) return;

        // send notification
        if( SystemNotifications::isSupported() )
        {
            if( !systemNotifications_ )
            {

                const Application& application( *static_cast<const Application*>( Singleton::get().application() ) );
                systemNotifications_ = new SystemNotifications( this, application.applicationName(), application.applicationIcon() );
            }
            systemNotifications_->processMessage( tr( "Network monitor" ), QString( tr( "Monitoring network traffic on device %1." ) ).arg( device ) );
        }

        device_ = device;
        XmlOptions::get().set( "NETWORK_DEVICE", device_ );

    }

    //__________________________________________________________
    void NetworkMonitor::_updateConfiguration( void )
    {
        Debug::Throw( "Network::NetworkMonitor::_updateConfiguration.\n" );

        // samples
        networkLoad_.setSampleSize( XmlOptions::get().get<int>( "SAMPLES" ) );

        // stop device timer
        if( deviceTimer_.isActive() ) deviceTimer_.stop();

        // device selection
        const bool automatic( XmlOptions::get().get<bool>( "NETWORK_AUTO_SELECT" ) );
        if( automatic )
        {

            _checkDevice( XmlOptions::get().raw( "NETWORK_DEVICE" ) );
            deviceTimer_.start( 1000*XmlOptions::get().get<int>( "NETWORK_AUTO_SELECT_RATE" ), this );

        } else {

            if( deviceTimer_.isActive() ) deviceTimer_.stop();
            _selectDevice( XmlOptions::get().raw( "NETWORK_DEVICE" ) );

        }

        // update menu
        networkMenu_->setDevice( device_ );
        networkMenu_->setIsAutomatic( automatic );

    }

}
